@@ -3,8 +3,7 @@ SMS Server Tools 3
 Copyright (C) 2006- Keijo Kasvi
 http://smstools3.kekekasvi.com/
 
-Based on SMS Server Tools 2 from Stefan Frings
-http://www.meinemullemaus.de/
+Based on SMS Server Tools 2, http://stefanfrings.de/smstools/
 SMS Server Tools version 2 and below are Copyright (C) Stefan Frings.
 
 This program is free software unless you got it under another license directly
@@ -44,7 +43,6 @@ void alarm_handler(int severity, char* format, ...)
   char text[1024];
   char cmdline[PATH_MAX+1024];
   char timestamp[40];
-  time_t now;
 
   if (_alarmhandler[0])
   {
@@ -53,8 +51,7 @@ void alarm_handler(int severity, char* format, ...)
     va_end(argp);
     if (severity<=_alarmlevel)
     {
-      time(&now);
-      strftime(timestamp,sizeof(timestamp),"%Y-%m-%d %H:%M:%S",localtime(&now));
+      make_datetime_string(timestamp, sizeof(timestamp), 0, 0, logtime_format);
       snprintf(cmdline,sizeof(cmdline),"%s ALARM %s %i %s \"%s\"",_alarmhandler,timestamp,severity, process_title, text);
       my_system(cmdline, "alarmhandler");
     }  
