@@ -1,10 +1,9 @@
 /*
 SMS Server Tools 3
-Copyright (C) Keijo Kasvi
+Copyright (C) 2006- Keijo Kasvi
 http://smstools3.kekekasvi.com/
 
-Based on SMS Server Tools 2 from Stefan Frings
-http://www.meinemullemaus.de/
+Based on SMS Server Tools 2, http://stefanfrings.de/smstools/
 SMS Server Tools version 2 and below are Copyright (C) Stefan Frings.
 
 This program is free software unless you got it under another license directly
@@ -24,7 +23,12 @@ Either version 2 of the License, or (at your option) any later version.
 void set_alarmhandler(char* handler,int level);
 
 // calls the alarm handler
-void alarm_handler0(int severity, char *devicename, char *text);
-void alarm_handler(int severity,char* devicename,char* format, ...);
+void alarm_handler0(int severity, char *text);
+
+#ifdef __GNUC__
+void alarm_handler(int severity, char* format, ...) __attribute__ ((format(printf, 2, 3)));
+#else
+void alarm_handler(int severity, char* format, ...);
+#endif
 
 #endif
