@@ -3,8 +3,7 @@ SMS Server Tools 3
 Copyright (C) 2006- Keijo Kasvi
 http://smstools3.kekekasvi.com/
 
-Based on SMS Server Tools 2 from Stefan Frings
-http://www.meinemullemaus.de/
+Based on SMS Server Tools 2, http://stefanfrings.de/smstools/
 SMS Server Tools version 2 and below are Copyright (C) Stefan Frings.
 
 This program is free software unless you got it under another license directly
@@ -28,9 +27,34 @@ Either version 2 of the License, or (at your option) any later version.
 
 int Filehandle = -1;
 int Level;
+int SavedLevel;
 int Filehandle_trouble = -1;
 char *trouble_logging_buffer = 0;
 int last_flush_was_clear = 1;
+unsigned long long trouble_logging_start_time = 0; // 3.1.16beta. For total time of continuous trouble.
+int logging_start_printed = 0; // 3.1.16beta.
+
+int change_loglevel(int new_level)
+{
+
+  SavedLevel = Level;
+  Level = new_level;
+
+  return SavedLevel;
+}
+
+void restore_loglevel()
+{
+
+  Level = SavedLevel;
+}
+
+// 3.1.14:
+int get_loglevel()
+{
+
+  return Level;
+}
 
 int openlogfile(char *filename, int facility, int level)
 {
@@ -71,13 +95,13 @@ int openlogfile(char *filename, int facility, int level)
     {
       result = Filehandle;
 
-      if (smart_logging && level < 7)
+      if (smart_logging) // 3.1.16beta. && level < 7)
       {
         char filename2[PATH_MAX];
         int error = 0;
         int i;
 
-        if (snprintf(filename2, sizeof(filename2), "%s", filename) >= sizeof(filename2))
+        if ((size_t)snprintf(filename2, sizeof(filename2), "%s", filename) >= sizeof(filename2))
           error = 1;
         else
         {
@@ -137,6 +161,7 @@ void closelogfile()
   }
 
   trouble_logging_started = 0;
+  trouble_logging_start_time = 0;
 }
 
 void writelogfile0(int severity, int trouble, char *text)
@@ -150,7 +175,6 @@ void writelogfile(int severity, int trouble, char* format, ...)
   char text[SIZE_LOG_LINE];
   char text2[SIZE_LOG_LINE];
   char timestamp[40];
-  time_t now;
 
   // make a string of the arguments
   va_start(argp,format);
@@ -172,8 +196,7 @@ void writelogfile(int severity, int trouble, char* format, ...)
     }
     else
     {
-      time(&now);
-      strftime(timestamp,sizeof(timestamp),"%Y-%m-%d %H:%M:%S",localtime(&now));
+      make_datetime_string(timestamp, sizeof(timestamp), 0, 0, logtime_format);
       snprintf(text2, sizeof(text2),"%s,%i, %s: %s\n", timestamp, severity, process_title, text);
 
       // 3.1.5:
@@ -184,65 +207,128 @@ void writelogfile(int severity, int trouble, char* format, ...)
     }
   }
 
-  if (smart_logging && Level < 7)
+  if (smart_logging) // 3.1.16beta. && Level < 7)
   {
+    // 3.1.16beta: Mark the start of communication:
+    if (!logging_start_printed)
+    {
+      logging_start_printed = 1;
+
+      make_datetime_string(timestamp, sizeof(timestamp), 0, 0, logtime_format);
+      snprintf(text2, sizeof(text2),"%s,%i, %s: Start:\n", timestamp, severity, process_title);
+      strcat_realloc(&trouble_logging_buffer, text2, 0);
+    }
+
     if (trouble)
     {
       if (!trouble_logging_started)
       {
         trouble_logging_started = 1;
 
+        // 3.1.16beta:
+        if (!trouble_logging_start_time)
+        {
+          if (put_command_sent)
+            trouble_logging_start_time = put_command_sent;
+          else
+            trouble_logging_start_time = time_usec();
+        }
+
         // Global process_id is the same as int device in many functions calls.
-        if (process_id >= 0)
-          statistics[process_id]->status = 't';
+        if (PROCESS_IS_MODEM)
+          STATISTICS->status = 't';
+
+        // 3.1.16beta: Mark the start of trouble (not when a process is starting, value 2):
+        if (trouble == 1)
+        {
+          make_datetime_string(timestamp, sizeof(timestamp), 0, 0, logtime_format);
+          snprintf(text2, sizeof(text2),"%s,%i, %s: Trouble:\n", timestamp, severity, process_title);
+          strcat_realloc(&trouble_logging_buffer, text2, 0);
+        }
+
       }
     }
 
     // Any message is stored:
-    time(&now);
-    strftime(timestamp, sizeof(timestamp), "%Y-%m-%d %H:%M:%S", localtime(&now));
+    make_datetime_string(timestamp, sizeof(timestamp), 0, 0, logtime_format);
     snprintf(text2, sizeof(text2),"%s,%i, %s: %s\n", timestamp, severity, process_title, text);
 
     if (text2[strlen(text2) -1] != '\n')
       strcpy(text2 +sizeof(text2) -5, "...\n");
 
-    if (!trouble_logging_buffer)
-    {
-      if ((trouble_logging_buffer = (char *)malloc(strlen(text2) +1)))
-        *trouble_logging_buffer = 0;
-    }
-    else
-      trouble_logging_buffer = (char *)realloc((void *)trouble_logging_buffer, strlen(trouble_logging_buffer) +strlen(text2) +1);
-
-    if (trouble_logging_buffer)
-      strcat(trouble_logging_buffer, text2);
+    strcat_realloc(&trouble_logging_buffer, text2, 0);
   }
 }
 
+int make_duration_string(char *dest, size_t size_dest, unsigned long long time_start, unsigned long long time_now)
+{
+  int duration;
+
+  *dest = 0;
+
+  duration = (int)(time_now - time_start) / 100000;
+  if (duration >= 10)
+  {
+    if (duration < 600)
+      snprintf(dest, size_dest, "%.1f sec.", (double)duration / 10);
+    else
+    {
+      duration = (int)duration / 10;
+      snprintf(dest, size_dest, "%i min %i sec.", (int)duration / 60, (int)(duration - duration / 60 * 60));
+    }
+
+    return 1;
+  }
+
+  return 0;
+}
+
 void flush_smart_logging()
 {
+  static unsigned long long last_flush_time = 0; // 3.1.16beta.
+  char text2[SIZE_LOG_LINE];
+  char timestamp[40];
+  char duration[64];
 
   if (trouble_logging_started && trouble_logging_buffer)
   {
-     write(Filehandle_trouble, trouble_logging_buffer, strlen(trouble_logging_buffer));
-     last_flush_was_clear = 0;
+    write(Filehandle_trouble, trouble_logging_buffer, strlen(trouble_logging_buffer));
+    last_flush_was_clear = 0;
+    last_flush_time = time_usec();
+
+    if (trouble_logging_start_time)
+    {
+      make_duration_string(duration, sizeof(duration), trouble_logging_start_time, last_flush_time);
+      make_datetime_string(timestamp, sizeof(timestamp), 0, 0, logtime_format);
+      snprintf(text2, sizeof(text2), "%s,%i, %s: (flush) %s\n", timestamp, LOG_NOTICE, process_title, duration);
+      write(Filehandle_trouble, text2, strlen(text2));
+    }
   }
   else
   {
     // 3.1.6: If some errors were printed and now all is ok, print it to the log:
     if (!last_flush_was_clear && Filehandle_trouble >= 0)
     {
-      char text2[SIZE_LOG_LINE];
-      char timestamp[40];
-      time_t now;
+      char trouble_time[128];
 
-      time(&now);
-      strftime(timestamp, sizeof(timestamp), "%Y-%m-%d %H:%M:%S", localtime(&now));
-      snprintf(text2, sizeof(text2), "%s,%i, %s: %s\n", timestamp, LOG_NOTICE, process_title, "Everything ok now.");
+      make_datetime_string(timestamp, sizeof(timestamp), 0, 0, logtime_format);
+
+      // 3.1.16beta:
+      //snprintf(text2, sizeof(text2), "%s,%i, %s: %s\n", timestamp, LOG_NOTICE, process_title, "Everything ok now.");
+      *trouble_time = 0;
+      if (trouble_logging_start_time && last_flush_time)
+        if (make_duration_string(duration, sizeof(duration), trouble_logging_start_time, last_flush_time))
+          snprintf(trouble_time, sizeof(trouble_time), " Duration of trouble was %s", duration);
+
+      snprintf(text2, sizeof(text2), "%s,%i, %s: %s%s\n", timestamp, LOG_NOTICE, process_title, "Everything ok now.", trouble_time);
       write(Filehandle_trouble, text2, strlen(text2));
     }
 
     last_flush_was_clear = 1;
+    logging_start_printed = 0;
+    trouble_logging_start_time = 0;
+    put_command_sent = 0;
+    last_flush_time = 0;
   }
 
   trouble_logging_started = 0;
@@ -253,3 +339,61 @@ void flush_smart_logging()
     trouble_logging_buffer = 0;
   }
 }
+
+// ******************************************************************************
+// Collect character conversion log, flush it if called with format==NULL.
+// Also prints to the stdout, if debugging.
+void logch(char* format, ...)
+{
+  va_list argp;
+  char text[2048];
+  int flush = 0;
+
+  if (format)
+  {
+    va_start(argp, format);
+    vsnprintf(text, sizeof(text), format, argp);
+    va_end(argp);
+
+    if (strlen(logch_buffer) +strlen(text) < sizeof(logch_buffer))
+    {
+      sprintf(strchr(logch_buffer, 0), "%s", text);
+      // Line wrap after space character:
+      // Outgoing conversion:
+      if (strlen(text) >= 3)
+        if (strcmp(text +strlen(text) -3, "20 ") == 0)
+          flush = 1;
+      // Incoming conversion:
+      if (!flush)
+        if (strlen(text) >= 6)
+          if (strcmp(text +strlen(text) -6, "20[ ] ") == 0)
+            flush = 1;
+      // Line wrap after a reasonable length reached:
+      if (!flush)
+        if (strlen(logch_buffer) > 80)
+          flush = 1;
+    }
+#ifdef DEBUGMSG
+  printf("%s", text);
+#endif
+  }
+  else
+    flush = 1;
+
+  if (flush)
+  {
+    if (*logch_buffer)
+      writelogfile(LOG_DEBUG, 0, "charconv: %s", logch_buffer);
+    *logch_buffer = 0;
+#ifdef DEBUGMSG
+  printf("\n");
+#endif
+  }
+}
+
+char prch(char ch)
+{
+  if ((unsigned char)ch >= ' ')
+    return ch;
+  return '.';
+}
