@@ -3,8 +3,7 @@ SMS Server Tools 3
 Copyright (C) 2006- Keijo Kasvi
 http://smstools3.kekekasvi.com/
 
-Based on SMS Server Tools 2 from Stefan Frings
-http://www.meinemullemaus.de/
+Based on SMS Server Tools 2, http://stefanfrings.de/smstools/
 SMS Server Tools version 2 and below are Copyright (C) Stefan Frings.
 
 This program is free software unless you got it under another license directly
@@ -32,8 +31,30 @@ int set_numberformat(int *numberformat, char *number, int number_type);
 // flash_sms enables the flash flag.
 // mode select the pdu version (old or new).
 // if udh is true, then udh_data contains the optional user data header in hex dump, example: "05 00 03 AF 02 01"
-void make_pdu(char* number, char* message, int messagelen, int alphabet, int flash_sms, int report, int udh,
-              char* udh_data, char* mode, char* pdu, int validity, int replace_msg, int system_msg, int number_type);
+void make_pdu(
+  char *number,
+  char *message,
+  int messagelen,
+  int alphabet,
+  int flash_sms,
+  int report,
+  int with_udh,
+  char *udh_data,
+  char *mode,
+  char *pdu,
+  int validity,
+  int replace_msg,
+  int system_msg,
+  int number_type,
+  char *smsc,
+  // 3.1.16beta2:
+  int message_reference,
+  int reject_duplicates,
+  int reply_path,
+  int sms_class,
+  int tp_dcs,
+  int ping
+);
 
 // Splits a PDU string into the parts 
 // Input: 
@@ -73,7 +94,16 @@ int remove_concatenation(char *udh);
 
 int explain_toa(char *dest, char *octet_char, int octet_int);
 
-int get_pdu_details(char *dest, int size_dest, char *pdu, int mnumber);
+void explain_status(char *dest, size_t size_dest, int status);
+
+int get_pdu_details(char *dest, size_t size_dest, char *pdu, int mnumber);
 void sort_pdu_details(char *dest);
 
+int pdu2text(char *pdu, char *text, int *text_length, int *expected_length,
+             int with_udh, char *udh, char *udh_type, int *errorpos);
+int text2pdu(char* text, int length, char* pdu, char* udh);
+
+int read_pdu_text(char *pdu, size_t pdu_size, char *text);
+int get_pdu_submit_to(char *to, size_t to_size, char *pdu);
+
 #endif
