@@ -1,10 +1,9 @@
 /*
 SMS Server Tools 3
-Copyright (C) Keijo Kasvi
+Copyright (C) 2006- Keijo Kasvi
 http://smstools3.kekekasvi.com/
 
-Based on SMS Server Tools 2 from Stefan Frings
-http://www.meinemullemaus.de/
+Based on SMS Server Tools 2, http://stefanfrings.de/smstools/
 SMS Server Tools version 2 and below are Copyright (C) Stefan Frings.
 
 This program is free software unless you got it under another license directly
@@ -26,17 +25,25 @@ typedef struct
   int multiple_failed_counter;  // Number of multiple failed SM (3 consecutive ERROR's from modem)
   int usage_s;			// Modem usage to send SM in seconds
   int usage_r;			// Modem usage to receive SM in seconds
-  char status;			// s=send r=receive i=idle b=blocked -=not running
+  char status;			// s=send r=receive i=idle b=blocked -=not running t=trouble
+  int message_counter;          // This is a number of sent messages, stored in GSM1.counter file.
+                                // Note: first 3 counters are cleared every time when stats_interval is reached.
+  time_t last_init;
+  int ssi;
+  int ber;
 } _stats;
 
-_stats* statistics[DEVICES];	// Statistic data (shared memory!)
+_stats* statistics[NUMBER_OF_MODEMS]; // Statistic data (shared memory!)
 int rejected_counter;		// Statistic counter, rejected SM, number does not fit into any queue
 time_t start_time;		// Start time of smsd, allows statistic functions
 int printstatus;		// if 1 smsd outputs status on stdout
 time_t last_stats;		// time when the last stats file was created
-char d_stats[256];		// path to statistic files
+char d_stats[PATH_MAX];		// path to statistic files
 int stats_interval;		// time between statistic files in seconds.
 int stats_no_zeroes;		// Suppress files that contain only zeroes
+int status_interval;            // time between updating status file in seconds
+
+char *shared_buffer;
 
 /* Creates shared memory variables for statistic data */
 
@@ -62,5 +69,8 @@ void print_status();
 
 void checkwritestats();
 
+void update_message_counter(int messages, char *modemname);
+
+void write_status();
 
 #endif // STATS_H
