@@ -1,10 +1,9 @@
 /*
 SMS Server Tools 3
-Copyright (C) Keijo Kasvi
+Copyright (C) 2006- Keijo Kasvi
 http://smstools3.kekekasvi.com/
 
-Based on SMS Server Tools 2 from Stefan Frings
-http://www.meinemullemaus.de/
+Based on SMS Server Tools 2, http://stefanfrings.de/smstools/
 SMS Server Tools version 2 and below are Copyright (C) Stefan Frings.
 
 This program is free software unless you got it under another license directly
@@ -21,7 +20,4 @@ char whitelist[256];	// Filename of the white-list
 /* Used with >= 3.1x */
 int inwhitelist_q(char* msisdn, char *queuename);
 
-/* Used with < 3.1x */
-int inwhitelist(char* msisdn);
-
 #endif
